import javax.swing.*;

import java.awt.Color;
import java.awt.event.*;

public class ShuffleBoardFrame extends JFrame implements ActionListener {
	private static final long serialVersionUID = 1L;	// Meh
	
	private JMenuItem resetItem, undoItem, team1ColorItem, team2ColorItem, replayItem, exitItem;
	private JRadioButtonMenuItem fastItem, regularItem, slowItem;
	
	ShuffleBoardPanel panel;
	
	public ShuffleBoardFrame(String title) {
		super(title);
		
		JMenuBar menuBar = new JMenuBar();
		setJMenuBar(menuBar);
		
		JMenu speedMenu = new JMenu("Speed");
		speedMenu.add(fastItem 		= new JRadioButtonMenuItem("Fast"));
		speedMenu.add(regularItem 	= new JRadioButtonMenuItem("Regular"));
		speedMenu.add(slowItem 		= new JRadioButtonMenuItem("Slow"));
		
		fastItem.addActionListener(this);
		regularItem.addActionListener(this);
		slowItem.addActionListener(this);
		
		ButtonGroup group = new ButtonGroup();
		group.add(fastItem);
		group.add(regularItem);
		group.add(slowItem);
		
		JMenu gameMenu = new JMenu("Game");
		gameMenu.add(resetItem 		= new JMenuItem("Reset"));
		gameMenu.add(undoItem 		= new JMenuItem("Undo"));
		gameMenu.add(team1ColorItem = new JMenuItem("Team 1 Colour"));
		gameMenu.add(team2ColorItem = new JMenuItem("Team 2 Colour"));
		gameMenu.add(speedMenu);		
		gameMenu.add(replayItem 	= new JMenuItem("Replay"));
		gameMenu.add(exitItem 		= new JMenuItem("Exit"));
		menuBar.add(gameMenu);
		
		resetItem.addActionListener(this);
		undoItem.addActionListener(this);
		team1ColorItem.addActionListener(this);
		team2ColorItem.addActionListener(this);
		replayItem.addActionListener(this);
		exitItem.addActionListener(this);
		
		setContentPane(panel = new ShuffleBoardPanel());
		
		regularItem.doClick();
				
		setSize(ShuffleBoardPanel.BoardWidth,ShuffleBoardPanel.BoardHeight);
		setLocation(100,100);
		setResizable(false);	// Sorry no resizing
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		update();
	}
	
	public void update() {
		repaint();	// Repaint the window
	}

	public void actionPerformed(ActionEvent e) {
	    if (e.getSource() == resetItem) {
	    	panel.initialize();
	        update();
	    }
	    else if (e.getSource() == undoItem) {
	    	panel.undoMove(false);
	        update();
	    }
	    else if (e.getSource() == team1ColorItem)
	    	handleColorChange("Team1");
	    else if (e.getSource() == team2ColorItem)
	    	handleColorChange("Team2");
	    else if (e.getSource() == replayItem) {
	    	panel.undoMove(true);
	    	update();
	    }
	    else if (e.getSource() == exitItem)
	    	System.exit(0);
	    else if (e.getSource() == fastItem) {
	    	Weight.setFriction(0.99);
	    	update();
	    }
	    else if (e.getSource() == regularItem) {
	    	Weight.setFriction(0.98);
	    	update();
	    }
	    else if (e.getSource() == slowItem) {
	    	Weight.setFriction(0.97);
	    	update();		
	    }
	}
	
	public void handleColorChange(String team) {
		if (team.equals("Team1")) {
			Color color = JColorChooser.showDialog(
                    this,                   // The parent window
                    "Choose a Color",       // Title on Dialog Box
                    Color.RED);             // Initial color selected
			// If the selected color isn't the same as the other team's color, change it
			if (color != null && !color.equals(panel.getTeam2Color())) {
				panel.setTeam1Color(color);
			}
		}
		else if (team.equals("Team2")) {
			Color color = JColorChooser.showDialog(
                    this,                   // The parent window
                    "Choose a Color",       // Title on Dialog Box
                    Color.RED);             // Initial color selected
			// If the selected color isn't the same as the other team's color, change it
			if (color != null && !color.equals(panel.getTeam1Color())) {
				panel.setTeam2Color(color);
			}
		}
		update();
		panel.update();
	}
}
