import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.Timer;

public class ShuffleBoardPanel extends JPanel implements ActionListener, MouseListener, MouseMotionListener{
	private static final long serialVersionUID = 1L;	// Meh
	
	public static final int BoardWidth  = 300; 	// The Board Width
	public static final int BoardHeight = 800;  // The Board Height
	public static final int Border		= BoardHeight / 40; 	// The Board border/boundary
	
	// The walls
	private int leftWall 	= Border;
	private int top 		= Border;
	private int rightWall	= BoardWidth - Border;
	private int bottom		= BoardHeight - 70;
	
	// Score lines, distance from top
	private int firstLine	= top + 350;
	private int secondLine	= top + 200;
	private int thirdLine	= top + 75;
		
	private Weight 	selectedWeight 	= null;		// Weight being shot
	private Point 	shuffleHead 	= null;		// The head of the line of 'shuffling'
	private Point 	shuffleTail 	= null;		// The tail of the line of 'shuffling'
	
	private ArrayList<Weight> Weights = new ArrayList<Weight>();
	
	// Collections for replay button
	private ArrayList<Weight> orderShot		= new ArrayList<Weight>();
	private ArrayList<Point> shuffleHeads 	= new ArrayList<Point>();
	private ArrayList<Point> shuffleTails	= new ArrayList<Point>();
		
	private boolean justAdded = true;	// If we just added the Weight's locations to their position collection
	
	private boolean curveShot = false;	// If they are doing a curve shot
	
	private Point	curvePoint = null;	// Where they clicked to do the curve shot
	
	private Color team1;
	private Color team2;
	
	private Image anImage;
	
	private ClassLoader url = this.getClass().getClassLoader();
		
	private CollisionSet collisions = new CollisionSet();
	
	//Timer used to draw motion
    private int milliseconds = 2; //time between timer events
    private Timer timer = new Timer(milliseconds, this); 
    	
	public Color getTeam1Color() { return team1; }
	public Color getTeam2Color() { return team2; }
	
	public void setTeam1Color(Color color) { team1 = color; }
	public void setTeam2Color(Color color) { team2 = color; }
	
	public ShuffleBoardPanel() {
		anImage = Toolkit.getDefaultToolkit().createImage(url.getResource("background750.jpg"));
		
		team1 = Color.red;
		team2 = Color.blue;
				
		initialize();
		addEventHandlers();
		timer.start();
		
		update();
	}
	
	public void initialize() {
		// Clear the collections
		Weights.clear();
		orderShot.clear();
		shuffleHeads.clear();
		shuffleTails.clear();
		
		// Position the Weights at the bottom of the screen, equal distance apart
		
		int startX = Border + Weight.RADIUS + 3;
		int startY = BoardHeight - Border - 100;
		int displacementX =  (2 * Weight.RADIUS) + 7;
		
		// Team 1
		Weights.add(new Weight(new Point(startX, startY), team1));
		Weights.add(new Weight(new Point(startX + displacementX, startY), team1));
		Weights.add(new Weight(new Point(startX + (2 * displacementX), startY), team1));
		Weights.add(new Weight(new Point(startX + (3 * displacementX), startY), team1));
		
		// Team 2
		Weights.add(new Weight(new Point(startX + (4 * displacementX), startY), team2));
		Weights.add(new Weight(new Point(startX + (5 * displacementX), startY), team2));
		Weights.add(new Weight(new Point(startX + (6 * displacementX), startY), team2));
		Weights.add(new Weight(new Point(startX + (7 * displacementX), startY), team2));
		
	}
	
	
	public Weight weightTouching(Weight w) {
    	// Answer any Weight touching Weight w, null if there are none
		Point p = w.getLocation();
		for (Weight weight: Weights) {
        int distance = (p.x - weight.getLocation().x) * (p.x - weight.getLocation().x) +
                       (p.y - weight.getLocation().y) * (p.y - weight.getLocation().y);
        if ((distance <= 4*(Weight.RADIUS * Weight.RADIUS)) && (weight != w) )
            return weight;
       }
       return null;
    }
	
	public void checkBounds(Weight w) {
		// Stops the weight if it is outside the boundaries, and grays it out
		if (w.getLocationX() < leftWall || w.getLocationX() > rightWall || w.getLocationY() < top || w.getLocationY() > bottom) {
			w.setMoving(false);
			w.setvx(0.0);
			w.setvy(0.0);
			w.setColour(Color.gray);
			update();
		}
		else {
			// Reset color if in bounds
			for (int i = 0; i < 4; i++)
				if (Weights.get(i).equals(w)) {
					w.setColour(team1);
					return;
				}
			for (int i = 4; i < Weights.size(); i++)
				if (Weights.get(i).equals(w)) {
					w.setColour(team2);
					return;
				}
		}
	}
	
	public boolean isEverythingStopped() {
		// Checks if no Weights are moving
		for (Weight weight: Weights) {
			if (weight.isMoving())
				return false;
		}
		return true;
	}
	
	public int weightScore(Weight w) {
		// Gives the inputed weight's score depending on its location
		
		// If the weight is out of bounds, its score is 0
		if (w.getLocationX() < leftWall || w.getLocationX() > rightWall || w.getLocationY() < top || w.getLocationY() > bottom)
			return 0;
		
		// If the Weight is hanging off the end of table, but not more than halfway, score is 4
		if (w.getLocationY() > top && w.getLocationY() < top + Weight.RADIUS)
			return 4;
		
		// If the Weight is in between the third line and the edge of the table, score is 3
		if (w.getLocationY() < thirdLine - Weight.RADIUS)
			return 3;
		
		// If the Weight is between the third and second line, score is 2
		if (w.getLocationY() < secondLine - Weight.RADIUS)
			return 2;
		
		// If the Weight is between second line and first line, score is 1 
		if (w.getLocationY() < firstLine - Weight.RADIUS)
			return 1;
		
		// If none of the above (Weight is below first line)
		return 0;
	}
	
	public String scoringTeam() {
		// Returns the string name of the team that is scoring
		// That is, the team that has the highest up weight still in play
		
		// Set the current 'highest' location to be the bottom of the play field
		int highestTeam1 = bottom;
		int highestTeam2 = bottom;
		
		for (int i = 0; i < 4; i++) {
			Weight w = Weights.get(i);
			if (w.getLocationY() < highestTeam1 && w.getLocationY() > top)
				highestTeam1 = w.getLocationY();
		}
		for (int i = 4; i < Weights.size(); i++) {
			Weight w = Weights.get(i);
			if (w.getLocationY() < highestTeam2 && w.getLocationY() > top)
				highestTeam2 = w.getLocationY();
		}
		if (highestTeam1 < highestTeam2)
			return "Team1";
		else if (highestTeam2 < highestTeam1)
			return "Team2";
		
		// This happens if all the weights are at the same height
		else			
			return "None";
	}
	
	public int getCurrentScore() {
		// Returns the score of the team with the highest weight
		int score = 0;
		if (scoringTeam().equals("Team1")) {
			for (int i = 0; i < 4; i++) {
				score += weightScore(Weights.get(i));
			}
		}
		else if (scoringTeam().equals("Team2")) {
			for (int i = 4; i < Weights.size(); i++) {
				score += weightScore(Weights.get(i));
			}
		}
		return score;
	}
	
	public void undoMove(boolean replay) {
		// Undos the most recent move, moving all the Weights to their previous position
		for (Weight weight: Weights) {
			int size = weight.getPositions().size();
			if (size > 1) {
				weight.getPositions().remove(size - 1);
				weight.setLocation(weight.getPositions().get(size - 2));
			}
		}
		// If they pressed the replay button, first re-shoot the shot
		if (orderShot.size() >= 1 && shuffleHeads.size() >= 1 && shuffleTails.size() >= 1) {
			if (replay) {
				// Fire the shot again
				simulateMove();
			}
			// Prevents adding information twice
			orderShot.remove(orderShot.size() - 1);
			shuffleHeads.remove(shuffleHeads.size() -1 );
			shuffleTails.remove(shuffleTails.size() -1);
		}				
		update();
	}
	
	public void addEventHandlers() {
		addMouseListener(this);
		addMouseMotionListener(this);
	}
	
	public void removeEventHandlers() {
		removeMouseListener(this);
		removeMouseMotionListener(this);
	}

	// Unused event handlers
	public void mouseEntered(MouseEvent event) {}
	public void mouseExited(MouseEvent event) {}
	public void mouseMoved(MouseEvent event) {}
	public void keyTyped(KeyEvent event) {}
	public void keyReleased(KeyEvent event) {}
	public void mouseClicked(MouseEvent event) {}

	// Mouse press event handler
	public void mousePressed(MouseEvent event) {		
		    //start a new shot by making a shuffle stick
            selectedWeight = weightAt(event.getPoint());          
            shuffleHead = null; 
            shuffleTail = null;
            //make start of shuffle stick
            if(selectedWeight != null && curveShot == false)
            	// If not a curve shot, start shuffle stick at center of Weight
                shuffleHead = selectedWeight.getLocation();
            else if (selectedWeight != null && curveShot == true)
            	// If a curve shot, start shuffle stick at curvePoint
            	// Which was set to something in the weighAt method
            	shuffleHead = curvePoint;
	}

	// Mouse drag event handler
      public void mouseDragged(MouseEvent event) {
      	//stretch the shuffle stick to aim the shot
      	if (shuffleHead != null) shuffleTail = event.getPoint();
      	update();
    }

	// Mouse release event handler (i.e. stop dragging process)
    public void mouseReleased(MouseEvent event) {

    	//shoot the Weight based on direction and length of the shuffle stick
    	if((shuffleHead != null)&&(shuffleTail != null)) {
    		//determine horizontal and vertical speed from
    		//the length of the shuffle stick

    		double dx =  shuffleHead.x - shuffleTail.x;
    		double dy =  shuffleHead.y - shuffleTail.y;
          
    		
    		if(distanceBetween(shuffleHead, shuffleTail) > 3){
    			// If they tried a curve shot
    			if (curveShot) {
    				// Curve to the left if they clicked on the right
    				if (shuffleHead.x >= selectedWeight.getLocationX())
    					selectedWeight.setCurveDistance(distanceBetween(shuffleHead, selectedWeight.getLocation()));
    				// Curve to the right, if they clicked on the left
    				else
    					selectedWeight.setCurveDistance(distanceBetween(shuffleHead, selectedWeight.getLocation()) * -1);
        		}
    			//divide by 20, gives it a nice shuffleBoard look
    			selectedWeight.setvy(dy/20);
    			selectedWeight.setvx(dx/20);
    			selectedWeight.setMoving(true);
    			
    			// Save the following for the replay button
    			orderShot.add(selectedWeight);
    			shuffleHeads.add(new Point(shuffleHead.x, shuffleHead.y));
    			shuffleTails.add(new Point(shuffleTail.x, shuffleTail.y));
    		}
    		// Make it so we can now add all the Weights current positions
    		justAdded = false;
    		
    	}        
        //clear the shuffle stick after the shot
        selectedWeight = null;
        shuffleHead = null;
        shuffleTail = null;
        curvePoint = null;        
        
        update();
	}
    
    public void simulateMove() {
    	shuffleHead  = shuffleHeads.get(shuffleHeads.size() - 1);
    	shuffleTail  = shuffleTails.get(shuffleTails.size() - 1);
    	selectedWeight = orderShot.get(orderShot.size() - 1);
    	mouseReleased(null);	// Initiate a fake mouseReleased event
    }
        
	private double distanceBetween(Point p1, Point p2) {
        //answer the distance between two points
		return Math.sqrt ((double) ((p2.x - p1.x) *  (p2.x - p1.x) + (p2.y - p1.y) * (p2.y - p1.y)));               
	}
	
	// This is the Timer event handler 
    // Advance any Weights that are currently in motion
    public void actionPerformed(ActionEvent e) {
        
        //move each weight based on its current velocity
    	for(Weight weight: Weights) {

           // move the weight based on elapsed time in milliseconds
           // since it was last moved
    		weight.advance( milliseconds );
    		
    		/* I moved this outside the if statement (unlike the Tutorial code)
    		 * I realize this does wasteful loops every timer event but
    		 * it is the simplest way I could think of to constantly update
    		 * every individual weight's color. This way if the team's color
    		 * has been changed all of them get changed, and if a weight is
    		 * outside the boundaries, its individual color is gray
    		 */
    		checkBounds(weight);
    		if(weight.isMoving() ) {
    		 // Tutorial code:
           	 //check whether the weight has gone out of bounds
    			//checkBounds(weight);
           	 
           	 //Handle collisions (weights hitting other weights 
				Weight w = weightTouching(weight);             
				if (w != null ) collisions.add(weight,w);
			}
			collisions.removeOldCollisions();
               
    	}
    	update();
	}
	
	public void paintComponent(Graphics aPen) {
		// Overwrites the JPanel's paintComponent method
		super.paintComponent(aPen);
		
		aPen.drawImage(anImage, 0, 0, null);
		
		//Draw boundary
		aPen.setColor(Color.white);
        aPen.drawRect(Border,Border, BoardWidth - 40, BoardHeight - 90); 
        
        // Draw score lines
        aPen.drawLine(Border, firstLine, rightWall, firstLine);
        aPen.drawLine(Border, secondLine, rightWall, secondLine);
        aPen.drawLine(Border, thirdLine, rightWall, thirdLine);
        
        aPen.drawString("1", BoardWidth/2, secondLine + 20);
        aPen.drawString("2", BoardWidth/2, secondLine - 15);
        aPen.drawString("3", BoardWidth/2, thirdLine - 15);
				
		//Draw Shadows for the Weights
        for (int i = 0; i < Weights.size(); i++) {
        	Weight weight = Weights.get(i);
        	weight.drawShadowWith(aPen);
        }	    

        //Draw Weights
        for (int i = 0; i < Weights.size(); i++) {
        	Weight weight = Weights.get(i);
        	weight.drawWith(aPen);
        }
        
        // Draw the weight's scores if their team is scoring
        // Also set the current pen color to be the winning team's color
        if (scoringTeam().equals("Team1")) {
        	for (int i = 0; i < 4; i++) {
        		Weights.get(i).drawScore(aPen);
        	}
        	aPen.setColor(team1);
        }
        else if (scoringTeam().equals("Team2")) {
        	for (int i = 4; i < Weights.size(); i++) {
        		Weights.get(i).drawScore(aPen);
        	}
        	aPen.setColor(team2);
        }
        
        // Draw the current score
        aPen.drawString("Score: " + String.valueOf(getCurrentScore()), BoardWidth - 75, 15);
                
        //Draw the shuffle stick
        aPen.setColor(Color.black);
        if((shuffleHead != null)&&(shuffleTail != null)) {
          
           //aPen.drawLine(shuffleHead.x, shuffleHead.y, shuffleTail.x, shuffleTail.y);
           aPen.drawLine(shuffleHead.x, shuffleHead.y, shuffleTail.x+3, shuffleTail.y-3);
           aPen.drawLine(shuffleHead.x, shuffleHead.y, shuffleTail.x-3, shuffleTail.y+3);
        }        
	}
	
	public void update() {
		// If none of the Weights are moving and we haven't already added their current positions
		// Add their position and set their score
		if (isEverythingStopped() && !justAdded) {
			for (Weight weight: Weights) {
				weight.getPositions().add(new Point(weight.getLocationX(),weight.getLocationY()));
				weight.setScore(weightScore(weight));
			}
			justAdded = true;
		}
		
		// Otherwise just set their score
		else {
			for (Weight weight: Weights)
				weight.setScore(weightScore(weight));
		}
		
		removeEventHandlers();
		repaint();
		addEventHandlers();
	}
	
	// Return the Weight at the given location (if one exists)
	public Weight weightAt(Point p) {
		for (Weight weight: Weights) {
			int distance = (p.x - weight.getLocation().x) * (p.x - weight.getLocation().x) +
                       	   (p.y - weight.getLocation().y) * (p.y - weight.getLocation().y);
			// If they selected the colored part of the weight, this will shoot it straight
			if (distance <= 0.5*(Weight.RADIUS * Weight.RADIUS)) {
				curveShot = false;
				return weight;
			}
			else if (distance <= 2 * (Weight.RADIUS * Weight.RADIUS)) {
				curveShot = true;
				curvePoint = new Point(p);
				return weight;
			}
		}
       return null;
    }
}
