import java.awt.*;
import java.util.ArrayList;

// This class represents a Weight
public class Weight {
	public static final int RADIUS = ShuffleBoardPanel.BoardWidth / 25;
	
	// Instance variables
	private Color color;	// The Weights colour in the center
	
	boolean moving = false;	// Indicates if the Weight is moving or not
	
	private double x;		// x location of the Weight
	private double y;		// y location of the Weight
	
	private double curveDistance = 0.0;	// If they do a curve shot, the distance from the center to the clicked spot 
	
	private int score;		// The individual weight's score
	
	private static double friction;	// Changes depending on the user set Speed of the weights
	
	private Point location = new Point(0,0);	// x-y location of the Weight
	
	// ArrayList that each Weight stores its current position at after each shot
	private ArrayList<Point> positions = new ArrayList<Point>();
	
	//Horizontal and Vertical Velocities
    //vx<0 means left, vx>0 means right; vy<0 means up, vy>0 means down 
    private double vx; //horizontal velocity in pixels/millisecond
    private double vy; //vertical velocity in pixels/millisecond
    
    public static double getFriction() { return friction; }
    public static void setFriction(double newFriction) { friction = newFriction; }
    
    public void setScore(int newScore) { score = newScore; }
    
    public void setCurveDistance(double newCurve) { curveDistance = newCurve; }
    
    public double getCurveDistance(){ return curveDistance; }
    
    public Weight(Point aPoint, Color teamColor) {
    	color = teamColor;
		location = aPoint;
		x = location.getX();
		y = location.getY();
		
		friction = 0.98;	// Regular friction
		
		positions.add(new Point(location.x, location.y));

		//make the Weight stopped
        vx = 0.0;
        vy = 0.0;
        moving = false;
    }
    
 // The get & set methods
	public Color getColour() { return color; }
	public Point getLocation() { return location; }
	public int getLocationX() {return location.x;}
	public int getLocationY() {return location.y;}
	
	public void setColour(Color newColor) { color = newColor; }
	
	public void setLocation(Point aPoint) { 
	     location.x = aPoint.x;
	     location.y = aPoint.y;
	      x = aPoint.getX(); y = aPoint.getY(); }
    
    public void setLocation(int theX, int theY) { 
        location.x = theX;
        location.y = theY;
        x = location.getX(); 
        y = location.getY();
        }
    
    public void setMoving(boolean isMoving) { moving = isMoving; }
    public boolean isMoving() {return moving; } //answer whether weight is moving
    
    public ArrayList<Point> getPositions() { return positions; }
    
    //get and set the velocities of the weight
    public double getvx() {return vx;} //get horizontal velocity
    public double getvy() {return vy;} //get vertical velocity
    public void setvx( double v ) {vx = v; moving = ((vx != 0.0) || (vy != 0.0)); }
    public void setvy( double v ) {vy = v; moving = ((vx != 0.0) || (vy != 0.0));}
    
    public void stop() { 
    	vx = 0; 
    	vy = 0; 
    	moving = false; 
    }    
  
    public double velocity() {
    	//return the magnitude of the overall velocity 
    	return Math.sqrt ((vx * vx) + (vy * vy));
    }
    
    public void advance( int milliseconds) {
    	//This is the main method used by the timer event handler. It moves
        //the weight based on its current location and velocity
            	
    	if (moving) {
    		//Weight is moving so adjust its location based on its 
    		//velocity and the number of milliseconds since it was last moved
    		
    		// If the player initiated a curve shot, curveDistance should not be 0
    		// If they fired a straight shot, curveDistance will be 0
    		
    		x = x + (curveDistance/10)*Math.sin(milliseconds*50) + (vx*milliseconds);
    		y = y + (vy*milliseconds);
    		location.x = (int) x;
    		location.y = (int) y;
                  
    		//Apply some sort of friction to the moving weights
    		//so that they slow down over time
                  
    		//distribute friction loss based on horizontal and vertical 
    		//velocities
                                
    		vx = vx * friction; //slow down due to friction
    		vy = vy * friction; //slow down due to friction
                         
    		//if the Weight is moving very slowly declare it stopped
    		if( (Math.abs(vx) < 0.12) && (Math.abs(vy) < 0.12) ) {
                     moving = false;
                     vx = 0.0; //set velocity to zero
                     vy = 0.0;
                     curveDistance = 0.0;
    		}
    		
    	}
    	// Do nothing if the Weight isn't moving
    }
    
    // Draw the Weight using the given Graphics object
	public void drawWith(Graphics aPen) {		
		// Draw the Weight
		aPen.setColor(Color.gray);
		aPen.fillOval(location.x - Weight.RADIUS, location.y - Weight.RADIUS, Weight.RADIUS * 2, Weight.RADIUS * 2);	
		
		//Draw the colored center of Weight
		aPen.setColor(color);
		aPen.fillOval(location.x - (Weight.RADIUS/2), location.y - (Weight.RADIUS/2), Weight.RADIUS, Weight.RADIUS);					   

		// Draw a black border around the Ball
		aPen.setColor(Color.black);
		aPen.drawOval(location.x - Weight.RADIUS, location.y - Weight.RADIUS, Weight.RADIUS * 2, Weight.RADIUS * 2);
	}
	
	// Draw the Weight using the given Graphics object
	public void drawShadowWith(Graphics aPen) {
		//Draw the shadow of ball offset 3 pixels
		aPen.setColor(Color.red.darker().darker().darker());
		aPen.fillOval(location.x - Weight.RADIUS + 4, location.y - Weight.RADIUS + 4, 
					   Weight.RADIUS * 2, Weight.RADIUS * 2);
	}
	
	// Draw the score of the Weight if it's team is currently scoring
	public void drawScore(Graphics aPen) {
		aPen.setColor(Color.white);
		aPen.drawString(String.valueOf(score), location.x + 5, location.y + 5);
	}

}
